#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: sw=4:ts=4:expandtab

""" A Python logging library with super powers """

import sys
import itertools as it

from os import getcwd, path as p
from argparse import RawTextHelpFormatter, ArgumentParser

import pygogo as gogo

HDLRS_FULL = tuple(h for h in dir(gogo.handlers) if h.endswith("hdlr"))
HDLRS = tuple(h[:-5] for h in HDLRS_FULL)
LEVELS = ("critical", "error", "warning", "info", "debug")
FRMTRS_FULL = tuple(f for f in dir(gogo.formatters) if f.endswith("formatter"))
FORMATS = tuple(f[:-10] for f in FRMTRS_FULL)
CURDIR = p.basename(getcwd())
LOGFILE = "%s.log" % CURDIR

parser = ArgumentParser(
    description="description: Logs a given message",
    prog="gogo",
    usage="%(prog)s [options] <message>",
    formatter_class=RawTextHelpFormatter,
)

parser.add_argument(
    dest="message",
    nargs="?",
    default=sys.stdin,
    help="The message to log (default: reads from stdin).",
)

parser.add_argument(
    "-l",
    "--level",
    metavar="LEVEL",
    choices=LEVELS,
    default="info",
    help=(
        "The level to log the message (default: info).\n"
        "Must be one of: %s,\n%s.\n\n" % (", ".join(LEVELS[:4]), ", ".join(LEVELS[4:]))
    ),
)

parser.add_argument(
    "-n", "--name", default=CURDIR, help="The logger name (default: %s).\n\n" % CURDIR
)

parser.add_argument(
    "-D",
    "--high-hdlr",
    metavar="HANDLER",
    choices=HDLRS,
    default="stderr",
    help=(
        "The high pass log handler (default: stderr).\n"
        "Must be one of: %s,\n%s.\n\n" % (", ".join(HDLRS[:4]), ", ".join(HDLRS[4:]))
    ),
)

parser.add_argument(
    "-d",
    "--low-hdlr",
    metavar="HANDLER",
    choices=HDLRS,
    default="stdout",
    help=(
        "The low pass log handler (default: stdout).\n"
        "Must be one of: %s,\n%s.\n\n" % (", ".join(HDLRS[:4]), ", ".join(HDLRS[4:]))
    ),
)

parser.add_argument(
    "-L",
    "--high-level",
    metavar="LEVEL",
    choices=LEVELS,
    default="warning",
    help=(
        "Min level to log to the high pass handler\n"
        "(default: warning). Must be one of: %s,\n%s.\n\n"
        % (", ".join(LEVELS[:1]), ", ".join(LEVELS[1:]))
    ),
)

parser.add_argument(
    "-e",
    "--low-level",
    metavar="LEVEL",
    choices=LEVELS,
    default="debug",
    help=(
        "Min level to log to the low pass handler\n"
        "(default: debug). Must be one of: %s,\n%s.\n\n"
        % (", ".join(LEVELS[:1]), ", ".join(LEVELS[1:]))
    ),
)

parser.add_argument(
    "-F",
    "--high-format",
    metavar="FORMAT",
    choices=FORMATS,
    default="basic",
    help=(
        "High pass handler log format (default: basic)."
        "\nMust be one of: %s,\n%s.\n\n"
        % (", ".join(FORMATS[:4]), ", ".join(FORMATS[4:]))
    ),
)

parser.add_argument(
    "-o",
    "--low-format",
    metavar="FORMAT",
    choices=FORMATS,
    default="basic",
    help=(
        "Low pass handler log format (default: basic)."
        "\nMust be one of: %s,\n%s.\n\n"
        % (", ".join(FORMATS[:4]), ", ".join(FORMATS[4:]))
    ),
)

parser.add_argument(
    "-m",
    "--monolog",
    action="store_true",
    default=False,
    help="Log high level events only to high pass handler.\n\n",
)

parser.add_argument(
    "-f",
    "--filename",
    action="append",
    default=[LOGFILE],
    help=(
        "The filename to log to  (default: %s).\nUsed in the following "
        "handlers: file.\n\n"
    )
    % LOGFILE,
)

parser.add_argument(
    "-s",
    "--subject",
    default=["You've got mail"],
    action="append",
    help=(
        "The log subject (default: You've got mail)."
        "\nUsed in the following handlers: email.\n\n"
    ),
)

parser.add_argument(
    "-u",
    "--url",
    action="append",
    default=[""],
    help="The log url. Required for the following handlers:\nwebhook.\n\n",
)

parser.add_argument(
    "-H",
    "--host",
    default=["localhost"],
    action="append",
    help=(
        "The host (default: localhost).\nUsed in the following handlers: "
        "socket and syslog.\n\n"
    ),
)

parser.add_argument(
    "-p",
    "--port",
    metavar="NUM",
    type=int,
    action="append",
    default=[""],
    help=(
        "The port number (default: Python logging default).\nUsed in the "
        "following handlers: socket and syslog.\n\n"
    ),
)

parser.add_argument(
    "-t",
    "--tcp",
    action="count",
    default=0,
    help=(
        "Use TCP instead of UDP.\nUsed in the following handlers: socket and "
        "syslog.\n\n"
    ),
)

parser.add_argument(
    "-g",
    "--get",
    action="count",
    default=0,
    help=(
        "Use a GET request instead of POST.\nUsed in the following handlers: "
        "webhook.\n\n"
    ),
)

parser.add_argument(
    "-v", "--version", help="Show version and exit.", action="store_true", default=False
)

parser.add_argument(
    "-V",
    "--verbose",
    help="Increase output verbosity.",
    action="store_true",
    default=False,
)


def run():
    """CLI runner
    """
    args = parser.parse_args()
    gogo_logger = gogo.Gogo(__name__, verbose=args.verbose).get_logger("run")

    if args.version:
        gogo_logger.info("gogo v%s" % gogo.__version__)
        exit(0)

    counted = {"get", "tcp"}
    appended = {"filename", "subject", "url", "host", "port"}
    items = args._get_kwargs()
    counted_args = [i for i in items if i[0] in counted]
    appended_args = [i for i in items if i[0] in appended]

    high_appended_args = [(k, v[0]) for k, v in appended_args]
    high_counted_args = [(k, v > 0) for k, v in counted_args]
    high_kwargs = dict(it.chain(high_appended_args, high_counted_args))

    low_appended_args = [(k, v[-1]) for k, v in appended_args]
    low_counted_args = [(k, v > 1) for k, v in counted_args]
    low_kwargs = dict(it.chain(low_appended_args, low_counted_args))

    high_hdlr = getattr(gogo.handlers, "%s_hdlr" % args.high_hdlr)
    low_hdlr = getattr(gogo.handlers, "%s_hdlr" % args.low_hdlr)
    high_format = getattr(gogo.formatters, "%s_formatter" % args.high_format)
    low_format = getattr(gogo.formatters, "%s_formatter" % args.low_format)

    nkwargs = {
        "verbose": args.verbose,
        "high_level": args.high_level.upper(),
        "low_level": args.low_level.upper(),
        "high_formatter": high_format,
        "low_formatter": low_format,
        "monolog": args.monolog,
        "high_hdlr": high_hdlr(**high_kwargs),
        "low_hdlr": low_hdlr(**low_kwargs),
    }

    logger = gogo.Gogo(args.name, **nkwargs).get_logger("runner")

    try:
        message = args.message.read()
    except AttributeError:
        message = args.message

    getattr(logger, args.level)(message)
    exit(0)


if __name__ == "__main__":
    run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: sw=4:ts=4:expandtab

"""
tests.test
~~~~~~~~~~

Provides scripttests pygogo CLI functionality.
"""

import sys

sys.path.append("../pygogo")
import pygogo as gogo  # noqa

from difflib import unified_diff  # noqa
from os import path as p  # noqa
from io import StringIO  # noqa
from timeit import default_timer as timer  # noqa

from scripttest import TestFileEnvironment  # noqa


def main(script, tests, verbose=False, stop=True):
    """ Main method

    Returns 0 on success, 1 on failure
    """
    failures = 0
    logger = gogo.Gogo(__name__, verbose=verbose).logger
    short_script = p.basename(script)
    env = TestFileEnvironment(".scripttest")

    start = timer()

    for pos, test in enumerate(tests):
        num = pos + 1
        opts, arguments, expected = test
        joined_opts = " ".join(opts) if opts else ""
        joined_args = '"%s"' % '" "'.join(arguments) if arguments else ""
        command = "%s %s %s" % (script, joined_opts, joined_args)
        short_command = "%s %s %s" % (short_script, joined_opts, joined_args)
        result = env.run(command, cwd=p.abspath(p.dirname(p.dirname(__file__))))
        output = result.stdout

        if isinstance(expected, bool):
            text = StringIO(output).read()
            outlines = [str(bool(text))]
            checklines = StringIO(str(expected)).readlines()
        elif p.isfile(expected):
            outlines = StringIO(output).readlines()

            with open(expected, encoding="utf-8") as f:
                checklines = f.readlines()
        else:
            outlines = StringIO(output).readlines()
            checklines = StringIO(expected).readlines()

        args = [checklines, outlines]
        kwargs = {"fromfile": "expected", "tofile": "got"}
        diffs = "".join(unified_diff(*args, **kwargs))
        passed = not diffs

        if not passed:
            failures += 1
            msg = "ERROR! Output from test #%i:\n  %s\n" % (num, short_command)
            msg += "doesn't match:\n  %s\n" % expected
            msg += diffs if diffs else ""
        else:
            logger.debug(output)
            msg = "Scripttest #%i: %s ... ok" % (num, short_command)

        logger.info(msg)

        if stop and failures:
            break

    time = timer() - start
    logger.info("%s" % "-" * 70)
    end = "FAILED (failures=%i)" % failures if failures else "OK"
    logger.info("Ran %i scripttests in %0.3fs\n\n%s" % (num, time, end))
    sys.exit(failures)


if __name__ == "__main__":
    parent_dir = p.abspath(p.dirname(p.dirname(__file__)))
    script = p.join(parent_dir, "bin", "gogo")

    tests = [
        (["--help"], [], True),
        (["--version"], [], "gogo v%s\n" % gogo.__version__),
        ([], ["hello world"], "hello world\n"),
        (["-l debug"], ["hello world"], ""),
        (["-Vl debug"], ["hello world"], "hello world\n"),
    ]

    main(script, tests)



# -*- coding: utf-8 -*-
# vim: sw=4:ts=4:expandtab

"""
pygogo.handlers
~~~~~~~~~~~~~~~

Log handlers

Examples:
    Add a stdout handler::

        >>> logger = logging.getLogger()
        >>> logger.addHandler(stdout_hdlr())
        >>> logger.info('hello world')
        hello world

Attributes:
    ENCODING (str): The module encoding
"""

import sys
import logging
import socket

from os import environ
from logging import handlers as hdlrs

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse

ENCODING = "utf-8"

module_hdlr = logging.StreamHandler(sys.stdout)
module_logger = logging.getLogger(__name__)
module_logger.addHandler(module_hdlr)


def stdout_hdlr(**kwargs):
    """A standard output log handler

    Returns:
        New instance of :class:`logging.StreamHandler`

    Examples:
        >>> stdout_hdlr()  # doctest: +ELLIPSIS
        <...StreamHandler...>
    """
    return logging.StreamHandler(sys.stdout)


def stderr_hdlr(**kwargs):
    """A standard error log handler

    Returns:
        New instance of :class:`logging.StreamHandler`

    Examples:
        >>> stderr_hdlr()  # doctest: +ELLIPSIS
        <...StreamHandler...>
    """
    return logging.StreamHandler(sys.stderr)


def fileobj_hdlr(f, **kwargs):
    """A file object log handler

    Args:
        f (obj): A file like object.

    Returns:
        New instance of :class:`logging.StreamHandler`

    Examples:
        >>> from io import StringIO
        >>> fileobj_hdlr(StringIO())  # doctest: +ELLIPSIS
        <...StreamHandler...>
    """
    return logging.StreamHandler(f)


def file_hdlr(filename, mode="a", encoding=ENCODING, delay=False, **kwargs):
    """A file log handler

    Args:
        filename (string): The logfile name.

        mode (string): The file open mode (default: a, i.e., append).

        encoding (string): The file encoding (default: the module ENCODING).

        delay (bool): Defer file opening until the first call to emit
            (default: False).

    Returns:
        New instance of :class:`logging.FileHandler`

    Examples:
        >>> from tempfile import NamedTemporaryFile
        >>> f = NamedTemporaryFile()
        >>> file_hdlr(f.name)  # doctest: +ELLIPSIS
        <...FileHandler...>
    """
    fkwargs = {"mode": mode, "encoding": encoding, "delay": delay}
    return logging.FileHandler(filename, **fkwargs)


def socket_hdlr(host="localhost", port=None, tcp=False, **kwargs):
    """A socket log handler

    Args:
        host (string): The host name (default: localhost).

        port (int): The port (default: `logging.handlers` default).

        tcp (bool): Create a TCP connection instead of UDP (default: False).

    Returns:
        New instance of either :class:`logging.handlers.DatagramHandler` or
            :class:`logging.handlers.SocketHandler`

    Examples:
        >>> socket_hdlr()  # doctest: +ELLIPSIS
        <...DatagramHandler...>
        >>> socket_hdlr(tcp=True)  # doctest: +ELLIPSIS
        <...SocketHandler...>
    """
    if tcp:
        def_port = hdlrs.DEFAULT_TCP_LOGGING_PORT
        handler = hdlrs.SocketHandler
    else:
        def_port = hdlrs.DEFAULT_UDP_LOGGING_PORT
        handler = hdlrs.DatagramHandler

    address = (host, port or def_port)
    return handler(*address)


def syslog_hdlr(host="localhost", port=None, tcp=False, **kwargs):
    """A syslog log handler

    Args:
        host (string): The host name (default: localhost). Set to None to use
            the platform dependent domain socket.

        port (int): The port (default: `logging.handlers` default).

        tcp (bool): Create a TCP connection instead of UDP (default: False).

    Returns:
        New instance of :class:`logging.handlers.SysLogHandler`

    Examples:
        >>> syslog_hdlr()  # doctest: +ELLIPSIS
        <...SysLogHandler...>
    """
    # http://stackoverflow.com/a/13874620/408556
    DEF_SOCKETS = {"linux2": "/dev/log", "darwin": "/var/run/syslog"}

    if tcp:
        def_port = hdlrs.SYSLOG_TCP_PORT
        socktype = socket.SOCK_STREAM
    else:
        def_port = hdlrs.SYSLOG_UDP_PORT
        socktype = socket.SOCK_DGRAM

    if kwargs.get("address"):
        address = kwargs["address"]
    elif host:
        address = (host, port or def_port)
    elif sys.platform in DEF_SOCKETS:
        address = DEF_SOCKETS[sys.platform]
    else:
        msg = "Domain socket location for {} is not supported."
        raise ValueError(msg.format(sys.platform))

    if kwargs.get("facility"):
        facility = kwargs["facility"]
    elif kwargs.get("local_num") and 8 > kwargs["local_num"] >= 0:
        # http://unix.stackexchange.com/a/146993
        value = "LOG_LOCAL{}".format(kwargs["facility"])
        facility = getattr(hdlrs.SysLogHandler, value)
    else:
        facility = hdlrs.SysLogHandler.LOG_USER

    return hdlrs.SysLogHandler(address, facility=facility, socktype=socktype)


def buffered_hdlr(target=None, capacity=4096, level="error", **kwargs):
    """A memory buffered log handler

    Args:
        target (obj): The target logger handler (default stdout).

        capacity (int): The buffer size (default 4096).

        level (string): The min event level required to flush buffer
            (default: error).

    Returns:
        New instance of :class:`logging.handlers.MemoryHandler`

    Examples:
        >>> buffered_hdlr()  # doctest: +ELLIPSIS
        <...MemoryHandler...>
    """
    target = target or logging.StreamHandler(sys.stdout)
    return hdlrs.MemoryHandler(capacity, level.upper(), target)


def webhook_hdlr(url, **kwargs):
    """A web log handler

    Args:
        url (string): The logging endpoint.

    Kwargs:
        get (bool): Use a GET request instead of POST (default: False).

    Returns:
        New instance of :class:`logging.handlers.HTTPHandler`

    Examples:
        >>> webhook_hdlr('http://api.mysite.com/log')  # doctest: +ELLIPSIS
        <...HTTPHandler...>
    """
    parsed = urlparse(url)
    secure = parsed.scheme == "https"
    method = "GET" if kwargs.get("get") else "POST"
    args = (parsed.netloc, parsed.path)

    try:
        hdlr = hdlrs.HTTPHandler(*args, method=method, secure=secure)
    except TypeError:
        hdlr = hdlrs.HTTPHandler(*args, method=method)

    return hdlr


def email_hdlr(subject="You've got mail", host="localhost", port=587, **kwargs):
    """An email log handler

    Kwargs:
        subject (str): The email subject (default: You've got mail.).
        host (str): The email server host (default: localhost).
        port (str): The email sever port (default: 587).
        sender (str): The email sender (default: the system username at gmail).

        recipients (List[str]): The email recipients (default: the system
            username at gmail).

        username (str): The email sever username (default: None).
        password (str): The email sever password (default: None).

    Returns:
        New instance of :class:`logging.handlers.SMTPHandler`

    Examples:
        >>> email_hdlr('hello world')  # doctest: +ELLIPSIS
        <...SMTPHandler...>
    """
    address = (host, port) if port else host
    def_email = "%s@gmail.com" % environ.get("USER")
    sender = kwargs.get("sender", def_email)
    recipients = kwargs.get("recipients", [def_email])
    username = kwargs.get("username")
    password = kwargs.get("password")

    args = (address, sender, recipients, subject)
    credentials = (username, password) if username or password else None
    return hdlrs.SMTPHandler(*args, credentials=credentials)
